@@ -371,13 +371,14 @@ test_that("log_lik for zero-inflated and hurdle models runs without erros", {
 test_that("log_lik for ordinal models runs without erros", {
   ns <- 50
   nobs <- 8
-  ncat <- 4
+  nthres <- 3
+  ncat <- nthres + 1
   draws <- structure(list(nsamples = ns, nobs = nobs), class = "brmsdraws")
   draws$dpars <- list(
     mu = array(rnorm(ns * nobs), dim = c(ns, nobs)),
     disc = rexp(ns)
   )
-  draws$thres <- array(0, dim = c(ns, ncat - 1))
+  draws$thres$thres <- array(0, dim = c(ns, nthres))
   draws$data <- list(Y = rep(1:ncat, 2), ncat = ncat)
   draws$family$link <- "logit"
   
